pub fn set_planes_sequential_decoding_order(
order: &mut [usize],
erasures: &BTreeSet<usize>,
t: usize,
q: usize,
sub_chunk_no: usize,
)Expand description
Set the decoding order for planes based on intersection score
Planes (layers) are processed in order of increasing intersection score. This ensures that we can always recover the necessary U values for each layer.
Arguments
order- Output array to fill with layer indices in processing ordererasures- Set of erased chunk indicest- Number of y-sectionsq- Coupling factorsub_chunk_no- Total number of sub-chunks (α)